Class: OpenAssets::Transaction::SpendableOutput
- Inherits:
-
Object
- Object
- OpenAssets::Transaction::SpendableOutput
- Defined in:
- lib/openassets/transaction/spendable_output.rb
Overview
A transaction output with information about the asset ID and asset quantity associated to it.
Instance Attribute Summary collapse
-
#confirmations ⇒ Object
Returns the value of attribute confirmations.
-
#out_point ⇒ Object
An object that can be used to locate the output.
-
#output ⇒ Object
The actual output object.
Instance Method Summary collapse
-
#initialize(out_point, output) ⇒ SpendableOutput
constructor
A new instance of SpendableOutput.
Constructor Details
#initialize(out_point, output) ⇒ SpendableOutput
Returns a new instance of SpendableOutput.
16 17 18 19 |
# File 'lib/openassets/transaction/spendable_output.rb', line 16 def initialize(out_point, output) @out_point = out_point @output = output end |
Instance Attribute Details
#confirmations ⇒ Object
Returns the value of attribute confirmations.
12 13 14 |
# File 'lib/openassets/transaction/spendable_output.rb', line 12 def confirmations @confirmations end |
#out_point ⇒ Object
An object that can be used to locate the output.
8 9 10 |
# File 'lib/openassets/transaction/spendable_output.rb', line 8 def out_point @out_point end |
#output ⇒ Object
The actual output object.
10 11 12 |
# File 'lib/openassets/transaction/spendable_output.rb', line 10 def output @output end |